/*  zweic -- a compiler for Zwei
 *
 *  Stephane Micheloud & LAMP
 *
 *  $Id: Generator.scala 2697 2006-02-17 18:14:14 +0100 (Fri, 17 Feb 2006) michelou $
 */

package zweic;

class Generator(analyzer: Analyzer) {
  import RISC._;
  import scala.collection.mutable.{Map, HashMap};

  val code = new Code();

  private def genTmp(gen: Int => Unit) = {
    val tmpReg = code.getRegister();
    gen(tmpReg);
    code.freeRegister(tmpReg);
  }

  def gen(tree: Tree): Unit = tree match {
    case Program(classes, main) =>
      val init = code.getLabel();
      code.emit(BEQ, ZERO, init);

      // leave space for the VMTs
      // ...  complter ...

      // generate class and function definitions
      // ...  complter ...

      // generate main expression
      val start = code.getLabel();
      code.anchorLabel(start);
      genTmp { tmpReg => genLoad(main, tmpReg) }

      // exit
      code.emit(RET, ZERO, "exit program");

      // the following code is placed here because the space that
      // it occupies can be used after initialization.

      code.anchorLabel(init);
      // initialise stack pointer
      code.emit(SYSCALL, SP, 0, SYS_GET_TOTAL_MEM_SIZE); 
      // initialise garbage collector:
      // the heap starts at init and 
      // its size is 2/3 * (total_mem_size - init) words
      val r1 = code.getRegister();
      val r2 = code.getRegister();
      val r3 = code.getRegister();
      emitLoadConstant(r1, init.getAnchor());
      code.emit(SUB, r2, SP, r1); 
      code.emit(DIVIU, r2, r2, 3 * WORD_SIZE);
      code.emit(LSHI, r2, r2, 1); // r2 now contains the size of the heap
      emitLoadConstant(r3, SP << 27);
      code.emit(ADD, r2, r2, r3);
      code.emit(SYSCALL, r1, r2, SYS_GC_INIT);
      code.freeRegister(r3);
      code.freeRegister(r2);
      code.freeRegister(r1);

      // populate VMTs
      // ...  complter ...

      // jump to main expression
      code.emit(BEQ, ZERO, start, "jump to main expression");

    case ClassDef(_, _, members) =>
      // ...  complter ...

    case FieldDecl(_, _) => ;

    case method @ MethodDef(name, params, _, body) =>
      // ...  complter ...

    case While(cond, stats) =>
      // ...  complter ...

    case Var(varname, _, init) =>
      // ...  complter ...

    case Set(name, expr) =>
      // ...  complter ...

    case Do(expr) =>
      // ...  complter ...

    case PrintInt(expr) =>
      genTmp { tmpReg =>
        genLoad(expr, tmpReg);
        code.emit(SYSCALL, tmpReg, ZERO, SYS_IO_WR_INT, "printInt");
        code.emit(SYSCALL, ZERO, ZERO, SYS_IO_FLUSH, "flush");
      }
    case PrintChar(expr) =>
      // ...  complter ...

    case _ =>
      // ...  complter ...
  } // def gen(Tree): Unit

  def genLoad(tree: Expr, targetReg: Int): Unit = {
    def caseDefault(tree: Expr): Unit =
      genLoad(If(tree, IntLit(1), IntLit(0)), targetReg);

    assert(code.usedRegisters()(targetReg),
      "invariant violated: target register R"
      + targetReg + " not allocated");

    tree match {
      case NullLit() =>
        // ...  complter ...

      case IntLit(value) =>
        // ...  complter ...

      case ReadInt() =>
        // ...  complter ...

      case ReadChar() =>
        // ...  complter ...

      case Unop(op, expr) =>
        // ...  complter ...

      case Binop(op, left, right) =>
        // ...  complter ...

      case Block(stats, main) =>
        // ...  complter ...

      case If(cond, thenp, elsep) =>
        // ...  complter ...

      case Ident(name) =>
        // ...  complter ...

      case New(name, args) =>
        // ...  complter ...

      case Select(prefix, selector) =>
        // ...  complter ...

      case Call(receiver, method, args) =>
        // ...  complter ...

      case _ =>
        // ...  complter ...
    }
  } // genLoad

  private def emitLoadConstant(reg: Int, value: Int): Unit =
    emitLoadConstant(reg, value, null);

  private def emitLoadConstant(reg: Int, value: Int, s: String): Unit = {
    // ...  complter ...
  }

  /**
   * Generates code for conditions, that is constructs
   * which jump somewhere on the result of a test.
   */
  private def genCond(tree: Expr, targetLabel: code.Label, when: Boolean): Unit = {
    // ...  complter ...
    tree match {
        case Binop(op, left, right) => op match {
          // ...  complter ...
        }

        case Unop(op, expr) =>
          // ...  complter ...

        case _ =>
          // ...  complter ...
    }
  } // genCond


}
