/*  zweic -- a compiler for Zwei
 *
 *  Stephane Micheloud & LAMP
 *
 *  $Id: Scanner.scala 2142 2005-11-13 16:42:28 +0100 (Sun, 13 Nov 2005) dragos $
 */

package zweic;

import java.io.{InputStream, IOException};


/**
 * This class implements the scanner of the Zwei compiler.
 */
class Scanner(in: InputStream) {
  import scala.collection.mutable.HashMap;
  import Tokens._;

  /**
   * This character represents the end of the input.
   */
  private val EOF_CH = java.lang.Character.MAX_VALUE;

  /**
   * A table that associates a keyword with its corresponding
   * token class.
   */
  private val keywords = new HashMap[String, Token]();
  keywords("class")     = CLASS;
  // ...  complter ...

  /** The current token class.
   */
  var token: Token = BAD;

  /** The pushed-back token.
   */
  var pushedBack: Option[Triple[Int, String, Token]] = None;

  /** The position of the first character of the current token.
   */
  var start: Int = 0;

  /**
   * The string representation of the current token. This variable
   * is only set if the current token class has several possible
   * textual representations.
   */
  var chars: String = _;

  /**
   * A buffer for constructing string representations of tokens.
   */
  private val buf = new StringBuffer();

  /**
   * The current character.
   */
  private var ch = ' ';

  /**
   * The line and the column of the current token.
   */
  private var line = 1;
  private var column = 0;

  nextCh;
  nextToken;

  /**
   * This method reads the next token and stores the token class
   * in variable 'token'. If the token representation is not unique
   * it will also leave a textual representation in variable 'chars'.
   * Whitespaces and comments are skipped by this method.
   */
  def nextToken: Unit = pushedBack match {
    case None => {
      // skip comments and whitespace characters

      // ...  complter ...

      // initialize the position of the current token
      start = Position.encode(line, column);
      // read the current token
      token = readToken
    }

    case Some(Triple(st, ch, tok)) =>
      this.start = st;
      this.token = tok;
      this.chars = ch;
      pushedBack = None;
  }

  /**
   * Return the next token in the stream. 
   */
  def peekAhead: Token = pushedBack match {
    case None =>
      val oldStart = start;
      val oldChars = chars;
      val oldToken = token;

      nextToken;
      val tok = token;
      pushedBack = Some(Triple(start, chars, token));

      this.start = oldStart;
      this.chars = oldChars;
      this.token = oldToken;
      tok

    case Some(Triple(_, _, tok)) =>
      tok
  }
  
  /**
   * Read the next token, store its representation (if its not
   *  unique) in variable 'chars' and return the token class.
   */
  private def readToken: Token = ch match {
    // ...  complter ...
  }

  /**
   * Returns a textual representation of the current token.
   */
  def representation = {
    val representation = token.toString();
    if (token == NUMBER || token == IDENT || token == STRING)
      representation + "(\"" + chars + "\")";
    else
      representation
  }

  /**
   * Puts the next character into 'ch' and updates the current position.
   */
  private def nextCh: Unit = {
    ch match {
      case EOF_CH =>
        return
      case '\n' =>
        column = 1;
        line = line + 1
      case _ =>
        column = column + 1
    }
    try {
      ch = readCh;
      oldch = if ((oldch == '\r') && (ch == '\n')) readCh else ch;
      ch = if (oldch == '\r') '\n' else oldch
    }
    catch {
      case e: IOException => Report.fail(start, e.getMessage());
    }
  }

  private def readCh: char = {
    val c = in.read();
    if (c < 0) EOF_CH else c.asInstanceOf[Char]
  }

  private var oldch: char = ' ';
}
